@@ -1101,7 +1101,7 @@ spi_atk_bridge_signal_listener (GSignalInvocationHint *signal_hint,
   CORBA_any any;
   CORBA_Object c_obj;
   char *sp = NULL;
-  AtkObject *ao;
+  AtkObject *ao = NULL;
   gint detail1 = 0, detail2 = 0;
   SpiAccessible *s_ao = NULL;
 #ifdef SPI_BRIDGE_DEBUG
@@ -1156,19 +1156,28 @@ spi_atk_bridge_signal_listener (GSignalInvocationHint *signal_hint,
   else if ((signal_query.signal_id == atk_signal_children_changed) && gobject)
     {
       detail1 = g_value_get_uint (param_values + 1);
-      ao = atk_object_ref_accessible_child (ATK_OBJECT (gobject), 
-			        	    detail1);
-      if (ao) 
+      gpointer child = g_value_get_pointer (param_values + 2);
+      if (ATK_IS_OBJECT (child))
+        {
+          ao = ATK_OBJECT (child);
+          g_object_ref (ao);
+        }
+      else if ((detail != NULL) && (strcmp (detail, "add") == 0))
+        {
+          ao = atk_object_ref_accessible_child (ATK_OBJECT (gobject), 
+                                                detail1);
+        }
+      if (ao)
         {
           s_ao = spi_accessible_new (ao);
           c_obj = BONOBO_OBJREF (s_ao);
           spi_atk_bridge_init_object (&any, ATK_OBJECT (gobject), &c_obj);
-	  g_object_unref (ao);
-	}
+          g_object_unref (ao);
+        }
       else
-	{
-	  spi_atk_bridge_init_nil (&any, ATK_OBJECT (gobject));
-	}
+        {
+          spi_atk_bridge_init_nil (&any, ATK_OBJECT (gobject));
+        }
     }
   else
     {
@@ -1296,6 +1305,10 @@ spi_atk_tidy_windows (void)
   gint i;
 
   root = atk_get_root ();
+
+  if (!root)
+    return;
+
   n_children = atk_object_get_n_accessible_children (root);
   for (i = 0; i < n_children; i++)
     {
